﻿'Author: Daniel Kachakil  (dani@kachakil.com)  Copyright 2009
Imports System.IO
Imports System.Net
Imports System.Security.Cryptography.X509Certificates

Public Class BaseEngine
    'Protected m_EngineSettings As EngineSettings
    Protected m_HttpSettings As HttpSettings

    Public Event DebugEvent(ByVal sender As Object, ByVal e As DebugEventArgs)

    ''' <summary>
    ''' Returns the HttpSettings of the instance
    ''' </summary>
    Public ReadOnly Property HttpSettings() As HttpSettings
        Get
            Return Me.m_HttpSettings
        End Get
    End Property

    ''' <summary>
    ''' Initializes an instance of the engine with the specified parameters
    ''' </summary>
    ''' <param name="httpSettings">The settings for the HTTP requests</param>
    Public Sub New(ByVal httpSettings As HttpSettings) ', ByVal engineSettings As EngineSettings)
        'Callback function that is called automatically when making SSL requests
        ServicePointManager.ServerCertificateValidationCallback = AddressOf ValidateServerCertificateCallback

        Me.m_HttpSettings = httpSettings
        'Me.m_EngineSettings = engineSettings
    End Sub

    ''' <summary>
    ''' Callback function that is called automatically when making SSL requests
    ''' </summary>
    ''' <returns>Always true to avoid certificate errors</returns>
    Private Shared Function ValidateServerCertificateCallback(ByVal sender As Object, ByVal cert As X509Certificate, _
        ByVal chain As X509Chain, ByVal sslPolicyErrors As Net.Security.SslPolicyErrors) As Boolean

        'Return (sslPolicyErrors = sslPolicyErrors.None)
        Return True
    End Function

    ''' <summary>
    ''' Returns a string with the parameters to be passed to the HTTP request through GET or POST
    ''' </summary>
    ''' <param name="injection">The string to be injected into the vulnerable parameter</param>
    ''' <returns>The parameter string in URL-encoded form</returns>
    Private Function GetParameterString(ByVal injection As String) As String
        Dim params As String = ""

        For Each key As String In Me.m_HttpSettings.HttpParams.AllKeys
            params += key + "=" + System.Web.HttpUtility.UrlEncode(Me.m_HttpSettings.HttpParams(key))
            If key = Me.m_HttpSettings.InjectableHttpParamName Then
                params += System.Web.HttpUtility.UrlEncode(injection)
            End If
            params += "&"
        Next
        params = params.Substring(0, params.Length - 1)

        Return params
    End Function

    Public Function GetInjectionResponse(ByVal injection As String) As InjectionResponse
        Dim url As String = Me.m_HttpSettings.TargetBaseUrl

        If Not String.IsNullOrEmpty(Me.m_HttpSettings.AppendStartText) Then
            injection = Me.m_HttpSettings.AppendStartText + injection
        End If

        If Not String.IsNullOrEmpty(Me.m_HttpSettings.AppendEndText) Then
            injection += Me.m_HttpSettings.AppendEndText
        End If

        'Add GET parameters to the URL
        If Not Me.m_HttpSettings.UsePostMethod Then
            If url.Contains("?") Then
                url += "&"
            Else
                url += "?"
            End If
            url += Me.GetParameterString(injection)
        End If

        Dim webReq As HttpWebRequest = CType(HttpWebRequest.Create(url), HttpWebRequest)
        webReq.Timeout = Me.m_HttpSettings.HttpTimeout

        'Cookies
        If Me.m_HttpSettings.Cookies IsNot Nothing AndAlso Me.m_HttpSettings.Cookies.Count > 0 Then
            webReq.CookieContainer = New CookieContainer()

            Dim cookieDomain As String = Me.m_HttpSettings.TargetBaseUrl.Split("/"c)(2).Split(":"c)(0)

            For Each key As String In Me.m_HttpSettings.Cookies.AllKeys
                webReq.CookieContainer.Add(New Cookie(key, Me.m_HttpSettings.Cookies(key), "/", cookieDomain))
            Next
        End If

        'Authentication
        If Me.m_HttpSettings.Authentication = HttpSettings.AuthenticationType.Basic OrElse Me.m_HttpSettings.Authentication = HttpSettings.AuthenticationType.Digest Then
            webReq.Credentials = New NetworkCredential(Me.m_HttpSettings.UserName, Me.m_HttpSettings.Password)
        ElseIf Me.m_HttpSettings.Authentication = HttpSettings.AuthenticationType.Ntlm Then
            webReq.Credentials = New NetworkCredential(Me.m_HttpSettings.UserName, Me.m_HttpSettings.Password, Me.m_HttpSettings.Domain)
        End If

        'Proxy
        If Not String.IsNullOrEmpty(Me.m_HttpSettings.ProxyAddress) AndAlso Me.m_HttpSettings.ProxyPort > 0 Then
            webReq.Proxy = New WebProxy(Me.m_HttpSettings.ProxyAddress, Me.m_HttpSettings.ProxyPort)
        End If

        Dim t As New Stopwatch
        t.Start()

        'Send POST parameters
        If Me.m_HttpSettings.UsePostMethod Then
            webReq.Method = "POST"
            webReq.ContentType = "application/x-www-form-urlencoded"

            Dim postData As String = Me.GetParameterString(injection)

            Dim encoding As New System.Text.ASCIIEncoding()
            Dim bytes As Byte() = encoding.GetBytes(postData)
            webReq.ContentLength = bytes.Length
            Dim reqStream As Stream = webReq.GetRequestStream()
            reqStream.Write(bytes, 0, bytes.Length)
            reqStream.Close()
        End If


        'Connect and get response
        Dim webResp As HttpWebResponse = Nothing
        Dim respStream As Stream = Nothing
        Dim strReader As StreamReader = Nothing

        Try
            webResp = CType(webReq.GetResponse(), HttpWebResponse)
        Catch ex As WebException When ex.Status = WebExceptionStatus.ProtocolError
            webResp = CType(ex.Response, HttpWebResponse)
        Catch ex As WebException
            webResp = CType(ex.Response, HttpWebResponse)
            Me.RaiseDebugEvent("HTTP ERROR: " + ex.Status.ToString + " - " + ex.Message)
        End Try

        t.Stop()

        Dim responseData As String = Nothing
        Try
            respStream = webResp.GetResponseStream()
            strReader = New StreamReader(respStream)
            responseData = strReader.ReadToEnd()

            If webResp IsNot Nothing Then webResp.Close()
            If respStream IsNot Nothing Then respStream.Close()
            If strReader IsNot Nothing Then strReader.Close()
        Catch ex As Exception
        End Try

        Dim statusCode As HttpStatusCode = 0
        If webResp IsNot Nothing Then statusCode = webResp.StatusCode
        Dim response As New InjectionResponse(injection, responseData, statusCode, t.Elapsed)

        Me.RaiseDebugEvent(response)

        Return response
    End Function

    ''' <summary>
    ''' Raises a DebugEvent with the specified parameters and writes the message to the debug standard output
    ''' </summary>
    ''' <param name="message">The message of the event</param>
    Protected Sub RaiseDebugEvent(ByVal message As String)
        RaiseEvent DebugEvent(Me, New DebugEventArgs(message))
    End Sub




    Protected Sub RaiseDebugEvent(ByVal injectionResponse As InjectionResponse)
        RaiseEvent DebugEvent(Me, New DebugEventArgs(injectionResponse))
    End Sub

    Public Function IsTrue(ByVal injection As String, Optional ByVal trueSignature As String = "", Optional ByVal falseSignature As String = "") As Boolean
        If String.IsNullOrEmpty(trueSignature) AndAlso String.IsNullOrEmpty(falseSignature) Then
            Throw New ArgumentException("At least one signature must be established")
        End If

        Dim result As String = Me.GetInjectionResponse(injection).Response
        If Not String.IsNullOrEmpty(trueSignature) AndAlso Not String.IsNullOrEmpty(falseSignature) Then
            If result.Contains(trueSignature) Then
                Return True
            ElseIf result.Contains(falseSignature) Then
                Return False
            Else
                Throw New ApplicationException("No signature found. Try to keep only one and remove the other.")
            End If
        Else
            If String.IsNullOrEmpty(trueSignature) Then
                Return Not result.Contains(falseSignature)
            Else
                Return result.Contains(trueSignature)
            End If
        End If
    End Function

End Class
