﻿'Author: Daniel Kachakil  (dani@kachakil.com)  Copyright 2009
Imports System.Text

Public Class TableInfo
    Private m_Name As String = ""
    Private m_Schema As String = ""
    Private m_TotalBytes As Integer = -1
    Private m_RowCount As Integer = -1
    Private m_RawXmlData As New StringBuilder
    Private m_RawXmlSchema As String = ""
    Private m_Columns As New List(Of ColumnInfo)

    Public ReadOnly Property Name() As String
        Get
            Return Me.m_Name
        End Get
    End Property

    Public ReadOnly Property Schema() As String
        Get
            Return Me.m_Schema
        End Get
    End Property

    Public Property TotalBytes() As Integer
        Get
            Return Me.m_TotalBytes
        End Get
        Set(ByVal value As Integer)
            Me.m_TotalBytes = value
        End Set
    End Property

    Public Property RowCount() As Integer
        Get
            Return Me.m_RowCount
        End Get
        Set(ByVal value As Integer)
            Me.m_RowCount = value
        End Set
    End Property


    Public ReadOnly Property FullName() As String
        Get
            If String.IsNullOrEmpty(Me.m_Schema) Then
                Return Me.m_Name
            Else
                Return Me.m_Schema + "." + Me.m_Name
            End If
        End Get
    End Property

    Public ReadOnly Property RawXmlData() As StringBuilder
        Get
            Return Me.m_RawXmlData
        End Get
    End Property

    Public Property RawXmlSchema() As String
        Get
            Return Me.m_RawXmlSchema
        End Get
        Set(ByVal value As String)
            Me.m_RawXmlSchema = value

            Me.m_Columns.Clear()
            If Not String.IsNullOrEmpty(value) Then
                Dim dt As DataTable = ParseDataTable(Me.m_RawXmlSchema)
                For Each col As DataColumn In dt.Columns
                    Me.m_Columns.Add(New ColumnInfo(col))
                Next
            End If
        End Set
    End Property

    Public ReadOnly Property Columns() As ObjectModel.ReadOnlyCollection(Of ColumnInfo)
        Get
            Return Me.m_Columns.AsReadOnly()
        End Get
    End Property

    Public ReadOnly Property SelectedColumns() As ObjectModel.ReadOnlyCollection(Of ColumnInfo)
        Get
            Dim selectedCols As New List(Of ColumnInfo)
            For Each col In Me.m_Columns
                If col.Selected Then selectedCols.Add(col)
            Next
            Return selectedCols.AsReadOnly()
        End Get
    End Property

    Public Sub New(ByVal schema As String, ByVal tableName As String)
        Me.m_Schema = schema
        Me.m_Name = tableName

        If Not String.IsNullOrEmpty(Me.m_Schema) AndAlso Me.m_Schema.Contains(" ") Then
            Me.m_Schema = "[" + m_Schema + "]"
        End If
        If Not String.IsNullOrEmpty(Me.m_Name) AndAlso Me.m_Name.Contains(" ") Then
            Me.m_Name = "[" + m_Name + "]"
        End If
    End Sub

    Public Function GetDataTable() As DataTable
        Dim dt As DataTable = ParseDataTable(Me.m_RawXmlData.ToString)

        If dt.Columns.Count > 0 AndAlso Me.m_Columns.Count > 0 Then
            dt.Columns.RemoveAt(0)
            For Each col As DataColumn In dt.Columns
                Dim i As Integer
                If Integer.TryParse(col.ColumnName.TrimStart("c"c), i) Then
                    col.ColumnName = Me.m_Columns(i).ColumnName()
                End If
            Next
        End If

        Return dt
    End Function

    Public Function GetSelectedColumnAliases() As String
        If Me.m_Columns.Count = 0 Then Return "*"

        Dim sb As New StringBuilder
        For i = 0 To Me.m_Columns.Count - 1
            Dim col As ColumnInfo = Me.m_Columns(i)
            If col.Selected Then
                sb.Append("[" + col.ColumnName + "]c" + i.ToString + ",")
            End If
        Next
        If sb.Length > 0 Then
            sb.Length -= 1
        Else
            sb.Append("null")
        End If

        Return sb.ToString
    End Function

    Public Shared Function ParseDataTable(ByVal xmlRawData As String) As DataTable
        If String.IsNullOrEmpty(xmlRawData) Then Return New DataTable

        Dim stream As New System.IO.StringReader("<ds>" + xmlRawData + "</ds>")
        Dim ds As New DataSet()
        ds.ReadXml(stream)

        Return ds.Tables(0)
    End Function
End Class
